=== modified file 'NEWS'
--- NEWS	2008-09-10 00:02:06 +0000
+++ NEWS	2008-09-24 04:52:40 +0000
@@ -7,10 +7,16 @@
       script. (Martin Albisetti)
 
     - Fixed broken template for project browsing with start-loggerhead
-      (Martin Albisett)
-      
-    - Added --reload options to restart the application when a python 
-      file change. (Guillermo Gonzalez) 
+      (Martin Albisetti)
+
+    - Added --reload options to restart the application when a python
+      file change. (Guillermo Gonzalez)
+
+    - Added error handling middleware. (Guillermo Gonzalez)
+
+    - Fix bug #243415 ("Tracebacks go to console but not log
+      file"). Also minor improvements to logging in serve-branches and
+      start-loggerhead. (Guillermo Gonzalez)
 
 1.6 [15Aug2008]
 ----------------

=== modified file 'loggerhead/apps/branch.py'
--- loggerhead/apps/branch.py	2008-09-09 22:47:45 +0000
+++ loggerhead/apps/branch.py	2008-09-24 04:54:18 +0000
@@ -2,6 +2,7 @@
 
 import logging
 import urllib
+import sys
 
 import bzrlib.branch
 import bzrlib.lru_cache
@@ -110,5 +111,9 @@
         try:
             c = cls(self, self.get_history())
             return c(environ, start_response)
+        except:
+            environ['exc_info'] = sys.exc_info()
+            environ['branch'] = self
+            raise
         finally:
             self.branch.unlock()

=== added file 'loggerhead/apps/error.py'
--- loggerhead/apps/error.py	1970-01-01 00:00:00 +0000
+++ loggerhead/apps/error.py	2008-09-29 21:35:53 +0000
@@ -0,0 +1,58 @@
+# Copyright (C) 2008  Guillermo Gonzalez <guillo.gonzo@gmail.com>
+#
+# This program is free software; you can redistribute it and/or modify
+# it under the terms of the GNU General Public License as published by
+# the Free Software Foundation; either version 2 of the License, or
+# (at your option) any later version.
+#
+# This program is distributed in the hope that it will be useful,
+# but WITHOUT ANY WARRANTY; without even the implied warranty of
+# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
+# GNU General Public License for more details.
+#
+# You should have received a copy of the GNU General Public License
+# along with this program; if not, write to the Free Software
+# Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
+#
+
+from loggerhead.controllers.error_ui import ErrorUI
+
+class ErrorHandlerApp(object):
+    """Class for WSGI error logging middleware."""
+
+    msg = " %s.%s: %s \n"
+
+    def __init__(self, application, **kwargs):
+        self.application = application
+
+    def __call__(self, environ, start_response):
+        try:
+            return self.application(environ, start_response)
+        except:
+            # test if exc_info has been set, in the case that
+            # the error is caused before BranchWSGGIApp middleware
+            if environ.has_key('exc_info') and environ.has_key('branch'):
+                # Log and/or report any application errors
+                return self.handle_error(environ, start_response)
+            else:
+                # simply propagate the error, this is logged
+                # by paste.httpexceptions.TransLogger middleware
+                raise
+
+    def handle_error(self, environ, start_response):
+        """Exception hanlder."""
+        self.log_error(environ)
+        return errapp(environ, start_response)
+
+    def log_error(self, environ):
+        exc_type, exc_object, exc_tb = environ['exc_info']
+        logger = environ['branch'].log
+        logger.exception(self.msg, exc_type.__module__,
+                              exc_type.__name__, exc_object)
+
+
+def errapp(environ, start_response):
+    '''Default (and trivial) error handling WSGI application.'''
+    c = ErrorUI(environ['branch'], environ['exc_info'])
+    return c(environ, start_response)
+

=== added file 'loggerhead/controllers/error_ui.py'
--- loggerhead/controllers/error_ui.py	1970-01-01 00:00:00 +0000
+++ loggerhead/controllers/error_ui.py	2008-09-29 21:32:30 +0000
@@ -0,0 +1,47 @@
+#
+# Copyright (C) 2008  Guillermo Gonzalez <guillo.gonzo@gmail.com>
+#
+# This program is free software; you can redistribute it and/or modify
+# it under the terms of the GNU General Public License as published by
+# the Free Software Foundation; either version 2 of the License, or
+# (at your option) any later version.
+#
+# This program is distributed in the hope that it will be useful,
+# but WITHOUT ANY WARRANTY; without even the implied warranty of
+# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
+# GNU General Public License for more details.
+#
+# You should have received a copy of the GNU General Public License
+# along with this program; if not, write to the Free Software
+# Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
+#
+
+from StringIO import StringIO
+import traceback
+
+from loggerhead.controllers import TemplatedBranchView
+from loggerhead import util
+
+class ErrorUI(TemplatedBranchView):
+
+    template_path = 'loggerhead.templates.error'
+
+    def __init__(self, branch, exc_info):
+        super(ErrorUI, self).__init__(branch, None)
+        self.exc_info = exc_info
+
+    def get_values(self, h, args, kw, headers):
+        exc_type, exc_object, exc_tb = self.exc_info
+        description = StringIO()
+        traceback.print_exception(exc_type, exc_object, None, file=description)
+        directory_breadcrumbs = (
+            util.directory_breadcrumbs(
+                self._branch.friendly_name,
+                self._branch.is_root,
+                'changes'))
+        return {
+            'branch': self._branch,
+            'error_title': 'An unexpected error occurred while proccesing the request:',
+            'error_description': description.getvalue(),
+            'directory_breadcrumbs': directory_breadcrumbs,
+        }

=== modified file 'loggerhead/history.py'
--- loggerhead/history.py	2008-09-09 22:47:45 +0000
+++ loggerhead/history.py	2008-09-27 05:13:24 +0000
@@ -341,8 +341,11 @@
             return revid
         if revid == 'head:':
             return self.last_revid
-        if self.revno_re.match(revid):
-            revid = self._revno_revid[revid]
+        try:
+            if self.revno_re.match(revid):
+                revid = self._revno_revid[revid]
+        except KeyError:
+            raise bzrlib.errors.NoSuchRevision(self._branch.nick, revid)
         return revid
 
     def get_file_view(self, revid, file_id):

=== added file 'loggerhead/templates/error.pt'
--- loggerhead/templates/error.pt	1970-01-01 00:00:00 +0000
+++ loggerhead/templates/error.pt	2008-09-29 21:32:30 +0000
@@ -0,0 +1,30 @@
+<!DOCTYPE html PUBLIC "-//W3C//DTD XHTML 1.0 Transitional//EN" "http://www.w3.org/TR/xhtml1/DTD/xhtml1-transitional.dtd">
+<tal:block define="onload string:load()">
+  <html xmlns="http://www.w3.org/1999/xhtml" metal:use-macro="macros/main">
+    <head>
+      <title metal:fill-slot="title"
+             tal:content="string:${branch/friendly_name} : error"></title>
+    </head>
+    <body>
+        <tal:block metal:fill-slot="heading">
+        <h1>
+           <tal:has-link condition="branch/branch_link">
+            <a tal:attributes="href branch/branch_link"
+               tal:content="branch/friendly_name">
+               nice/branch/name
+            </a>
+           </tal:has-link>
+            <tal:no-link condition="not: branch/branch_link">
+                <span metal:use-macro="breadcrumbs/directory"></span>
+            </tal:no-link>
+           : error
+        </h1>
+        <pre>
+            <p tal:content="error_title"/>
+            <p tal:content="error_description"/>
+        </pre>
+        <br/><br/>
+        </tal:block>
+    </body>
+  </html>
+</tal:block>

=== added file 'loggerhead/trace.py'
--- loggerhead/trace.py	1970-01-01 00:00:00 +0000
+++ loggerhead/trace.py	2008-09-09 02:34:35 +0000
@@ -0,0 +1,70 @@
+#
+# Copyright (C) 2008  Canonical Ltd.
+#                     (Authored by Martin Albisetti <argentina@gmail.com)
+# Copyright (C) 2008  Guillermo Gonzalez <guillo.gonzo@gmail.com>
+# Copyright (C) 2006  Robey Pointer <robey@lag.net>
+# Copyright (C) 2006  Goffredo Baroncelli <kreijack@inwind.it>
+#
+# This program is free software; you can redistribute it and/or modify
+# it under the terms of the GNU General Public License as published by
+# the Free Software Foundation; either version 2 of the License, or
+# (at your option) any later version.
+#
+# This program is distributed in the hope that it will be useful,
+# but WITHOUT ANY WARRANTY; without even the implied warranty of
+# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
+# GNU General Public License for more details.
+#
+# You should have received a copy of the GNU General Public License
+# along with this program; if not, write to the Free Software
+# Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
+#
+
+import os
+import logging
+import sys
+
+def make_handler(config, filename):
+    roll = config.get('log.roll', 'never')
+    if roll == 'daily':
+        h = logging.handlers.TimedRotatingFileHandler(filename, 'midnight', 1, 100)
+    elif roll == 'weekly':
+        h = logging.handlers.TimedRotatingFileHandler(filename, 'W0', 1, 100)
+    else:
+        h = logging.FileHandler(filename)
+    return h
+
+
+def setup_logging(log_folder, config, foreground):
+    # i hate that stupid logging config format, so just set up logging here.
+
+    if not os.path.exists(log_folder):
+        os.mkdir(log_folder)
+
+    f = logging.Formatter('%(levelname)-.3s [%(asctime)s.%(msecs)03d]'
+                          ' %(name)s: %(message)s','%Y%m%d-%H:%M:%S')
+    debug_log = make_handler(config, os.path.join(log_folder, 'debug.log'))
+    debug_log.setLevel(logging.DEBUG)
+    debug_log.setFormatter(f)
+    if foreground:
+        stdout_log = logging.StreamHandler(sys.stdout)
+        stdout_log.setLevel(logging.DEBUG)
+        stdout_log.setFormatter(f)
+    f = logging.Formatter('[%(asctime)s.%(msecs)03d] %(message)s',
+                          '%Y%m%d-%H:%M:%S')
+    access_log = make_handler(config, os.path.join(log_folder, 'access.log'))
+    access_log.setLevel(logging.INFO)
+    access_log.setFormatter(f)
+    
+    error_log = make_handler(config, os.path.join(log_folder, 'error.log'))
+    error_log.setLevel(logging.ERROR)
+    error_log.setFormatter(f)
+
+    logging.getLogger('').setLevel(logging.DEBUG)
+    logging.getLogger('').addHandler(debug_log)
+    logging.getLogger('loggerhead.access').addHandler(access_log)
+    logging.getLogger('loggerhead').setLevel(logging.ERROR)
+    logging.getLogger('loggerhead').addHandler(error_log)
+
+    if foreground:
+        logging.getLogger('').addHandler(stdout_log)

=== modified file 'serve-branches'
--- serve-branches	2008-09-10 00:02:06 +0000
+++ serve-branches	2008-09-24 04:52:40 +0000
@@ -29,6 +29,7 @@
 from loggerhead.apps.filesystem import (
     BranchesFromFileSystemRoot, UserBranchesFromFileSystemRoot)
 from loggerhead.util import Reloader
+from loggerhead.apps.error import ErrorHandlerApp
 
 
 def command_line_parser():
@@ -36,6 +37,7 @@
     parser.set_defaults(
         user_dirs=False,
         show_version=False,
+        log_folder=None,
         )
     parser.add_option("--user-dirs", action="store_true", dest="user_dirs",
                       help="Serve user directories as ~user.")
@@ -50,15 +52,14 @@
     parser.add_option("--reload", action="store_true", dest="reload",
                       help="Restarts the application when changing python"
                            " files. Only used for development purposes.")
+    parser.add_option('--log-folder', dest="log_folder",
+                      type=str, help="The directory to place log files in.")
     parser.add_option("--version", action="store_true", dest="show_version",
                       help="Print the software version and exit")
     return parser
 
 
 def main(args):
-    logging.basicConfig()
-    logging.getLogger().setLevel(logging.DEBUG)
-
     parser = command_line_parser()
     (options, args) = parser.parse_args(sys.argv[1:])
 
@@ -81,7 +82,7 @@
     if options.trunk_dir and not options.user_dirs:
         print "--trunk-dir is only valid with --user-dirs"
         sys.exit(1)
-        
+
     if options.reload:
         if Reloader.is_installed():
             Reloader.install()
@@ -95,8 +96,25 @@
         app = UserBranchesFromFileSystemRoot(path, options.trunk_dir)
     else:
         app = BranchesFromFileSystemRoot(path)
+
+    # setup_logging()
+    logging.basicConfig()
+    logging.getLogger('').setLevel(logging.DEBUG)
+    logger = getattr(app, 'log', logging.getLogger('loggerhead'))
+    if options.log_folder:
+        logfile_path = os.path.join(options.log_folder, 'serve-branches.log')
+    else:
+        logfile_path = 'serve-branches.log'
+    logfile = logging.FileHandler(logfile_path, 'a')
+    formatter = logging.Formatter('%(asctime)s %(levelname)-8s %(name)s:'
+                                  ' %(message)s')
+    logfile.setFormatter(formatter)
+    logfile.setLevel(logging.DEBUG)
+    logger.addHandler(logfile)
+    # setup_logging() #end
+    app = ErrorHandlerApp(app)
     app = HTTPExceptionHandler(app)
-    app = TransLogger(app)
+    app = TransLogger(app, logger=logger)
 
     if not options.user_prefix:
         prefix = '/'
@@ -109,7 +127,7 @@
         pass
     else:
         app = PrefixMiddleware(app, prefix=prefix)
-    
+
     if not options.user_port:
         port = '8080'
     else:

=== modified file 'serve-branches.1'
--- serve-branches.1	2008-09-10 00:02:06 +0000
+++ serve-branches.1	2008-09-10 23:12:59 +0000
@@ -21,11 +21,23 @@
 \fB\-\-trunk\-dir\fR=\fIDIR\fR
 The directory that contains the trunk branches.
 .TP
+\fB\-\-port\fR
+Port Loggerhead should listen on (defaults to 8080).
+.TP
+\fB\-\-host\fR
+Host Loggerhead should listen on.
+.TP
+\fB\-\-prefix\fR
+Specify host prefix.
+.TP
 \fB\-\-reload\fR
 Restarts the application when changing python files. Only used for development purposes.
 .TP
+\fB\-\-log\-folder\fR=\fILOG_FOLDER\fR
+The directory to place log files
+.TP
 \fB\-\-version\fR
-Print the software version and exit
+Print the software version and exit.
 .SH "LICENSE"
 GNU GPLv2 or later.
 .SH "SEE ALSO"

=== modified file 'start-loggerhead'
--- start-loggerhead	2008-08-11 20:09:44 +0000
+++ start-loggerhead	2008-09-09 03:47:24 +0000
@@ -31,48 +31,8 @@
 
 from loggerhead import daemon
 from loggerhead.apps.config import Root
-
-
-def make_handler(config, filename):
-    roll = config.get('log.roll', 'never')
-    if roll == 'daily':
-        h = logging.handlers.TimedRotatingFileHandler(filename, 'midnight', 1, 100)
-    elif roll == 'weekly':
-        h = logging.handlers.TimedRotatingFileHandler(filename, 'W0', 1, 100)
-    else:
-        h = logging.FileHandler(filename)
-    return h
-
-
-def setup_logging(log_folder, config, foreground):
-    # i hate that stupid logging config format, so just set up logging here.
-
-    if not os.path.exists(log_folder):
-        os.mkdir(log_folder)
-
-    f = logging.Formatter('%(levelname)-.3s [%(asctime)s.%(msecs)03d] %(name)s: %(message)s',
-                          '%Y%m%d-%H:%M:%S')
-    debug_log = make_handler(config, os.path.join(log_folder, 'debug.log'))
-    debug_log.setLevel(logging.DEBUG)
-    debug_log.setFormatter(f)
-    if foreground:
-        stdout_log = logging.StreamHandler(sys.stdout)
-        stdout_log.setLevel(logging.DEBUG)
-        stdout_log.setFormatter(f)
-    f = logging.Formatter('[%(asctime)s.%(msecs)03d] %(message)s',
-                          '%Y%m%d-%H:%M:%S')
-    access_log = make_handler(config, os.path.join(log_folder, 'access.log'))
-    access_log.setLevel(logging.INFO)
-    access_log.setFormatter(f)
-
-    logging.getLogger('').setLevel(logging.DEBUG)
-    logging.getLogger('').addHandler(debug_log)
-    logging.getLogger('wsgi').addHandler(access_log)
-
-    if foreground:
-        logging.getLogger('').addHandler(stdout_log)
-
-
+from loggerhead.trace import make_handler, setup_logging
+from loggerhead.apps.error import ErrorHandlerApp
 
 def main():
     home = os.path.realpath(os.path.dirname(__file__))
@@ -137,7 +97,8 @@
 
     app = app
     app = make_middleware(app)
-    app = make_filter(app, None)
+    app = make_filter(app, None, logger_name=log.name+'.access')
+    app = ErrorHandlerApp(app)
 
     if webpath:
         scheme, netloc, path, blah, blah, blah = urlparse.urlparse(webpath)

